using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using Godot;

namespace KeepersCompound.TMV.UI;

public partial class TextureBrowser : Node
{
	enum SortMode
	{
		Name,
		Family,
		Index,
		Count,
	}

	private Tree _folderTree;
	private LineEdit _searchBar;
	private MenuButton _sortMenu;
	private Button _openFolderButton;
	private MenuButton _browserMenu;
	private HFlowContainer _textureList;
	private TextureRect _previewTexture;
	private LineEdit _texturePath;

	public override void _Ready()
	{
		// !HACK TEMP
		Context.Instance.PathManager.Init("/stuff/Games/thief/drive_c/GOG Games/TG ND 1.27 (MAPPING)/");
		Context.Instance.CampaignName = "Endless Rain";

		_folderTree = GetNode<Tree>("%FolderTree");
		_searchBar = GetNode<LineEdit>("%SearchBar");
		_sortMenu = GetNode<MenuButton>("%SortMenu");
		_openFolderButton = GetNode<Button>("%OpenTextureFolderButton");
		_browserMenu = GetNode<MenuButton>("%BrowserOverflowMenu");
		_textureList = GetNode<HFlowContainer>("%TextureList");
		_previewTexture = GetNode<TextureRect>("%PreviewTexture");
		_texturePath = GetNode<LineEdit>("%PathBox");

		_searchBar.TextChanged += ApplySearchFilter;
		_sortMenu.GetPopup().IdPressed += ApplySortMode;

		BuildFolderTree();
		BuildTextureList(); // TODO: This should be triggered on folder change
	}

	private void BuildFolderTree()
	{
		_folderTree.Clear();

		var context = Context.Instance;
		var textureNames = context.PathManager.GetResourceNames(LGS.ResourceType.Texture, context.CampaignName);

		var treeItems = new Dictionary<string, TreeItem>();
		treeItems.Add("", _folderTree.CreateItem());
		foreach (var name in textureNames)
		{
			var tokens = name.Split('/');
			var fams = new string[tokens.Length + 1];
			fams[0] = "";
			for (var i = 1; i < tokens.Length; i++)
			{
				fams[i] = tokens[..i].Join("/");
				if (!treeItems.ContainsKey(fams[i]))
				{
					var item = _folderTree.CreateItem(treeItems[fams[i - 1]]);
					item.SetText(0, tokens[i - 1]);
					treeItems.Add(fams[i], item);
				}
			}
		}
	}

	private void BuildTextureList()
	{
		foreach (var child in _textureList.GetChildren())
		{
			child.QueueFree();
		}

		var pathManager = Context.Instance.PathManager;
		var campaign = Context.Instance.CampaignName;
		var resType = LGS.ResourceType.Texture;
		var textureNames = pathManager.GetResourceNames(resType, campaign);
		foreach (var name in textureNames)
		{
			var (_, path) = pathManager.GetResourcePath(resType, campaign, name);
			var texture = TextureLoader.LoadTexture(path);

			var textureRect = new TextureRect();
			textureRect.Texture = texture;
			textureRect.ExpandMode = TextureRect.ExpandModeEnum.IgnoreSize;
			textureRect.StretchMode = TextureRect.StretchModeEnum.KeepAspectCentered;
			textureRect.SetAnchorsPreset(Control.LayoutPreset.FullRect);

			var slot = new Panel();
			slot.Name = name;
			slot.CustomMinimumSize = new Vector2(128, 128);
			slot.AddChild(textureRect);
			slot.GuiInput += (input) =>
			{
				if (input is InputEventMouseButton mouseEvent &&
					mouseEvent.Pressed &&
					mouseEvent.ButtonIndex == MouseButton.Left)
				{
					SetPreviewTexture(name, texture);
				}
			};

			_textureList.AddChild(slot);
		}
	}

	private void SetPreviewTexture(string name, Texture2D texture)
	{
		_previewTexture.Texture = texture;
		_texturePath.Text = name;
	}

	private void ApplySearchFilter(string filter)
	{
		var regexString = "^.*" + Regex.Escape(filter).Replace("\\*", ".*") + ".*$";
		foreach (var node in _textureList.GetChildren())
		{
			var panel = (Panel)node;
			panel.Visible = Regex.IsMatch(panel.Name.ToString(), regexString);
		}
	}

	private void ApplySortMode(long id)
	{
		var popup = _sortMenu.GetPopup();
		for (var i = 0; i < popup.ItemCount; i++)
		{
			popup.SetItemChecked(i, popup.GetItemId(i) == id);
		}

		// TODO: Actualy sort
	}
}
